# (C) Copyright 2007-2019 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
""" Plugin finder for the current package. """


def get_plugins():
    """ Get the plugins from this package. """

    from .orange_plugin import OrangePlugin

    return [OrangePlugin()]

#### EOF #######################################################################
