# (C) Copyright 2007-2019 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!

"""
Version information for this Envisage distribution.

This file is autogenerated by the Envisage setup.py script.
"""
from __future__ import unicode_literals

#: The full version of the package, including a development suffix
#: for unreleased versions of the package.
version = "4.9.1"

#: The Git revision from which this release was made.
git_revision = "05bbaad61309d863bfd5723e8dd794dd1f655574"
